@@ -33,6 +33,7 @@ class Users(object):
 		self.email_password = None
 		self.smtp_host = "smtp.gmail.com"
 		self.smtp_port = 587
+		self.visible_email = None
 
 		# system variables.
 		self.verification_codes = {}
@@ -99,13 +100,13 @@ class Users(object):
 		# create.
 		try:
 			user = auth.create_user(
-			    email=email,
-			    email_verified=False,
-			    phone_number=phone_number,
-			    password=password,
-			    display_name=name,
-			    photo_url=photo_url,
-			    disabled=False)
+				email=email,
+				email_verified=False,
+				phone_number=phone_number,
+				password=password,
+				display_name=name,
+				photo_url=photo_url,
+				disabled=False)
 			success = True
 		except Exception as e: 
 			success = False
@@ -183,8 +184,17 @@ class Users(object):
 			return r3sponse.success_response(f"Successfully deleted user [{uid}].")
 		else:
 			return r3sponse.error_response(f"Failed to delete user [{uid}], error: {error}")
-	def iterate(self):
-		return auth.list_users().iterate_all()
+	def iterate(self, by_firestore=False):
+		if by_firestore:
+			response = self.firestore.load_collection("users/")
+			if not r3sponse.success(response): raise ValueError(response["error"])
+			users = []
+			emails = response["collection"]
+			for email in emails:
+				users.append(auth.get_user_by_email(email))
+			return users
+		else:
+			return auth.list_users().iterate_all()
 	def verify_id_token(self, id_token):
 		"""
 			Javascript:
@@ -221,7 +231,8 @@ class Users(object):
 
 		# check email.
 		try:
-			self.email
+			if self.email == None:
+				raise AttributeError("")
 		except AttributeError:
 			response = self.__initialize_email__()
 			if response["error"] != None: return response
@@ -246,7 +257,7 @@ class Users(object):
 			if response["error"] != None: return response
 
 		# success.
-		return r3sponse.success_response(f"Successfully send an email to user [{user.display_name}].")
+		return r3sponse.success_response(f"Successfully send an email to user [{user.email}].")
 
 		#
 	def send_code(self, 
@@ -267,7 +278,8 @@ class Users(object):
 
 		# check email.
 		try:
-			self.email
+			if self.email == None:
+				raise AttributeError("")
 		except AttributeError:
 			response = self.__initialize_email__()
 			if response["error"] != None: return response
@@ -301,7 +313,7 @@ class Users(object):
 			if response["error"] != None: return response
 
 		# success.
-		return r3sponse.success_response(f"Successfully send a verification code to user [{user.display_name}].")
+		return r3sponse.success_response(f"Successfully send a verification code to user [{user.email}].")
 
 		#
 	def verify_code(self, 
@@ -313,10 +325,7 @@ class Users(object):
 		# the message mode.
 		mode="verification",
 	):
-		# check mode.
-		options = list(self.verification_codes.keys())
-		if mode not in options:
-			return r3sponse.error_response(f"Selected an invalid mode [{mode}], valid options: {options}.")
+
 
 		# get user.
 		response = self.get(uid=uid, email=email)
@@ -368,7 +377,8 @@ class Users(object):
 			email=self.email_address,
 			password=self.email_password,
 			smtp_host=self.smtp_host,
-			smtp_port=self.smtp_port,)
+			smtp_port=self.smtp_port,
+			visible_email=self.visible_email,)
 		response = self.email.login()
 		if response["success"]:
 			return r3sponse.success_response("Successfully initialized the mail object.")
@@ -409,8 +419,23 @@ class FireStore(object):
 		if not doc.exists:
 			return r3sponse.error_response(f"Document [{reference}] does not exist.")
 		else:
-		    data = doc.to_dict()
-		    return r3sponse.success_response(f"Successfully loaded document [{reference}].", {"document":data})
+			data = doc.to_dict()
+			return r3sponse.success_response(f"Successfully loaded document [{reference}].", {"document":data})
+	def load_collection(self, reference):
+		doc = self.__get_doc__(reference)
+		try:
+			doc = doc.get()
+			success = True
+		except: success = False
+		if not success:
+			return r3sponse.error_response(f"Failed to load document [{reference}].")
+		if isinstance(doc, dict):
+			return r3sponse.error_response(f"Reference [{reference}] leads to a document, not a collection.")
+		data = []
+		for i in doc:
+			if i.exists:
+				data.append(i.id)
+		return r3sponse.success_response(f"Successfully loaded the document names of collection [{reference}].", {"collection":data, "documents":doc})
 	def save(self, reference, data):
 		doc = self.__get_doc__(reference)
 		try:
